/**
 * \file   seconsole.h
 * \author Piotr Koenig
 * \author Sander Elektronik AG
 * \date   2015.06.26
 * \copyright (C) 2015 Sander Elektronik AG. All rights reserved.
 * \brief  Definitions for console interface to host processor
 *
 */
#define PROG_VERSION    0x100
/* ===== Revision History: ================================================
Date       Ver   Who Why
---------- ----- --- ------------------------------------------------------
2019.12.18 1.00  PK  Start conversion for nrf52
2017.01.31 1.10  PK	 Change timing for UART connection detection - 2 SRM reads, wait up to 5s and repeat
2016.11.21 1.09  PK	 Respond to broadcast trace with random delay 0..8ms
2016.09.23 1.08  PK	 Make sure that after going to sleep, we are polling Pic with 99 
2016.09.22 1.07  PK	 Fix sending MAC on power-up, used address 0x99 intead of 99
2016.07.07 1.06  PK	 Do not poll SRM if could not read SRM info on startup
2016.06.22 1.05  PK  Add UART disable, before checking UART pins - for MBUS
2016.05.27 1.04  PK  Poll SRM, if no response, disable radio to save power, ret Vcc
2016.05.20 1.03  PK  Add configuration for changing operating channel
2016.02.08 1.02  PK  EEPROM params could be 0, check for type, dirty groups
2015.11.15 1.01  PK  Add rts to processRoutePacket
2015.06.26 1.00  PK  Start
===========================================================================*/
#ifndef _SECONSOLE_H_
#define _SECONSOLE_H_

#include <stdint.h>
#include "boards.h"
//#include "boards.h"
//#include "nrf.h"
//#include "app_uart.h"
//#include "app_error.h"
//#include "radio_test.h"
//#include "uart.h"
//#include "nrf51_bitfields.h"
//#include "radio_config.h"

#define REDIRECT_STDOUT
//#define ROM const
//#define code const

//typedef int8_t bool;

//#define ARR_SIZE(x) (sizeof(x)/sizeof(x[0]))

#define ARR_SIZE(a) (sizeof(a)/sizeof(a[0]))

#define STR(x) #x
#define XSTR(x) STR(x)
#define APP_VERSION "" XSTR(PROG_VERSION)
// Compiler version information.
#ifdef _MSC_VER
	#define COMPILER_VERSION XSTR(_MSC_VER)
#elif defined(__GNUC__)
	#define COMPILER_VERSION "" XSTR(__GNUC__) "." XSTR(__GNUC_MINOR__) "." XSTR(__GNUC_PATCHLEVEL__)
#elif defined(__ARMCC_VERSION)
	#define COMPILER_VERSION  XSTR(__ARMCC_VERSION)
#else
	#define COMPILER_VERSION "0"
#endif

#define OUT_STR_SIZE 0x220

//#define PROCESS_ESCAPES

enum  _if_state_ {
	 SM_IDLE
	//,SM_REFRESH_VALUES
	,SM_PRINT_HELP
	//,SM_PRINT_DIR
	//,SM_PRINT_DUMP
	//,SM_SERVER_TEST
	//,SM_POLL_SWITCHES
	//,SM_POLL_ADC
};

#define MAX_CMD_LEN 0x80
#define MAX_OUT_LEN 0x80

typedef struct _connection_ {
  //enum _if_state_ ifState;
  int8_t devType;
  int8_t cmdPtr;
  uint8_t helpIndex;
  int8_t tabPosition;
  int8_t tabCmdSel;
  int8_t retVal;
  int8_t printfEnable;
  uint16_t outPtr;
  uint16_t outSize;
  uint8_t stdoutPtr;
  //uint8_t tcpOutPtr;
  //uint16_t replyAddr;
  //int8_t cmdAckSent;
  char cmdStr[MAX_CMD_LEN];
  char outStr[MAX_OUT_LEN];
  char stdoutBuf[0x30];
  char searchStr[0x20];
}TCONN, *pTCONN;

typedef struct tcp_commands {
	char *cmd;
	uint8_t cmdLen;
	char *help;
	void (*function)(pTCONN ptc);
}TCMDS, *pTCMDS;

extern char outStr[OUT_STR_SIZE];
extern char ledStatusEnable;
int8_t processDeviceInput (pTCONN ptc, uint8_t i);
void initConsole(void);
char *getProgTitle(char *str);
uint32_t seatoi (char *ptr);
char *skipChars (char *ptr);
char *skipSpaces (char *ptr);
char *skipWord (char *ptr);
char *put_outstr(pTCONN ptc);
char *put_str(pTCONN ptc, char *data);
void put_crlf(pTCONN ptc);
char *put_array(pTCONN ptc, char *data, unsigned size);
bool put_ch(pTCONN ptc, char byte);
void put_flush(pTCONN ptc);
uint16_t put_get_space(pTCONN ptc);
char *convertControlChars(char *ptr);

void processRadioTest(pTCONN ptc);
void processDataRate(pTCONN ptc);
void processPower(pTCONN ptc);
void processStartChannel(pTCONN ptc);
void processEndChannel(pTCONN ptc);
void processDelay(pTCONN ptc);
void processRadioTx (pTCONN ptc);
void processRadioRx (pTCONN ptc);
void processConfiguration (pTCONN ptc);
void processRadioText (pTCONN ptc);
void processUartConfig (pTCONN ptc);
void processSrmTest (pTCONN ptc);
void processSrmTx (pTCONN ptc);
void processTemp (pTCONN ptc);
void processAdc (pTCONN ptc);
void processHelp (pTCONN ptc);
void processGetPic (pTCONN ptc);
void srmInit(void);
void srmPicRead(void);
void showByteHex(pTCONN ptc, uint8_t val);
void showByteDec(pTCONN ptc, uint8_t val);
void showWordDec(pTCONN ptc, uint16_t val);
void sendAck(pTCONN ptc);
char txRxSrm (char srmCount);
void txSrm (char srmCount);
void showDWordDec(pTCONN ptc, uint32_t val);
void showDWordHex(pTCONN ptc, uint32_t val);
void showWordHex(pTCONN ptc, uint16_t val);
void processPause (pTCONN ptc);
void srmTestError(void);
void processCommandTcp (pTCONN ptc);
void processBaud (pTCONN ptc);

extern uint8_t hexTab[];
extern uint8_t quietFlag, noTimeFlag;

extern uint8_t radioStopRequest;
extern volatile uint32_t srmMsecs, srmMsecsCurr, srmMsecsLast;
extern uint8_t rxActive;
//extern STDIO radio_stdio;
extern TCONN radioTc, uartTc;
extern uint8_t logRtPtr;
extern uint8_t picAddr, picEeprom[];
void timerInit(void);
extern uint8_t randomDelayCounter;


void uart_put(uint8_t cr);
void uart_putstr(const char *str);
void uart_putoutstr(void);
void uart_put_array (const char *str, uint8_t cnt);
void uart_put_crlf (void);
void seputs(const char *str);


//char *reverse(char s[]);
//char *itoa(int16_t n, char s[], char base);
//char *utoa(uint16_t n, char s[], char base);
//char *ltoa(long int n, char s[], char base);

void processUartInput (uint8_t cr);
int seTimerInit(void);

int initSpim(void);
char xferSpim(uint8_t const *tx, uint8_t *rx, uint8_t len);

#endif //ifndef _SECONSOLE_H_
